﻿// ReSharper disable MemberCanBePrivate.Global
// ReSharper disable MemberCanBeInternal
using anbs_cp.Interfaces;

namespace anbs_cp.Classes;

/// <summary>
/// Сериализованная модель значения класса, хранящего в себе состояния действия
/// </summary>
public sealed class ActionStateSerializableValue: ISerializable
{
    /// <summary>
    /// Значение - null?
    /// </summary>
    public bool IsNull { get; set; }

    /// <summary>
    /// Сериализуемое значение
    /// </summary>
    public string Serialized { get; set; } = string.Empty;

    /// <summary>
    /// Получает значение из <paramref name="value"/>
    /// </summary>
    /// <param name="value">Значение</param>
    /// <typeparam name="T">Тип значения</typeparam>
    internal void GetValue<T> (T? value)
    {
        // Значение - null?
        IsNull = value is null;

        // Если не null
        // - сериализую
        Serialized = value is not null ? new SysTextSerializer().Serialize(value) : string.Empty;
    }

    /// <summary>
    /// Устанавливает значение
    /// </summary>
    /// <typeparam name="T">Класс</typeparam>
    /// <returns>Значение</returns>
    internal T? SetValue<T> () => IsNull ? default : new SysTextSerializer().Deserialize<T>(Serialized);

    /// <summary>
    /// Получает значение из <paramref name="value"/>
    /// </summary>
    /// <param name="value">Значение</param>
    /// <typeparam name="T">Тип значения</typeparam>
    internal void GetSerializedValue<T> (T? value) where T : ISerializable, new()
    {
        // Значение - null?
        IsNull = value is null;

        // Если не null
        // - сериализую
        Serialized = value is not null ? value.Serialize() : string.Empty;
    }

    /// <summary>
    /// Устанавливает значение
    /// </summary>
    /// <typeparam name="T">Класс</typeparam>
    /// <returns>Значение</returns>
    internal T? SetSerializedValue<T> () where T : ISerializable, new()
    {
        // Если null
        if (IsNull)
            // - то возвращаем null
            return default;

        // Создаю модель
        T model = new ();

        // Десериализую модель
        model.Deserialize(Serialized);

        // Возвращаю модель
        return model;
    }

    #region Реализация интерфейса ISerializable
    /// <inheritdoc />
    public string Serialize () => new SysTextSerializer().Serialize(this);

    /// <inheritdoc />
    public void Deserialize (string json)
    {
        // Десериализую строку
        ActionStateSerializableValue item = new SysTextSerializer().Deserialize<ActionStateSerializableValue>(json) ?? new();

        // Передаю параметры
        IsNull = item.IsNull;
        Serialized = item.Serialized;
    }
    #endregion
}