﻿using anbs_cp.Exceptions;
using anbs_cp.Interfaces;

namespace anbs_cp.Classes;

/// <summary>
/// Состояние действия
///
/// Обновлено 2024.03.26
/// * Добавлен класс ActionState для классов, поддерживаемых интерфейс I
/// </summary>
public sealed class ActionStateSerialized<T>: ActionState<T> where T : ISerializable, new()
{
    /// <summary>
    /// Добавляет другое состояние (например, результат другого действия, который возвращает <see cref="ActionState"/>).
    /// </summary>
    /// <param name="state">Запись состояния</param>
    public void AddState (ActionStateSerialized<T> state)
    {
        base.AddState(state);
    }

    #region Переопределение реализации интерфейса ISerializable
    /// <summary>
    /// Сериализовать элемент в формат json
    /// </summary>
    /// <returns>Строка в формате json</returns>
    /// <exception cref="SerializeException">Ошибка при сериализации</exception>
    public new string Serialize ()
    {
        // Создаю модель
        ActionStateSerializable serializableModel = new();

        // Для каждого информационного сообщения
        foreach (ActionStateMessage info in Info)
            // - добавляю в модель
            serializableModel.Info.Add(info.Serialize());

        // Для каждого предупреждения
        foreach (ActionStateMessage warning in Warnings)
            // - добавляю в модель
            serializableModel.Info.Add(warning.Serialize());

        // Для каждой ошибки
        foreach (ActionStateMessage error in Errors)
            // - добавляю в модель
            serializableModel.Info.Add(error.Serialize());

        // Создаю модель значения
        ActionStateSerializableValue value = new();

        // Получаю данные
        value.GetSerializedValue(Value);

        // Добавляю в модель
        serializableModel.Value = value.Serialize();

        // Возвращаю сериализованную модель
        return new SysTextSerializer().Serialize(serializableModel);
    }

    /// <summary>
    /// Восстановить элемент из формата json
    /// </summary>
    /// <param name="json">Строка в формате json</param>
    /// <exception cref="SerializeException">Ошибка при десериализации</exception>
    public new void Deserialize (string json)
    {
        // Десериализую строку
        ActionStateSerializable itemSerializable = new SysTextSerializer().Deserialize<ActionStateSerializable>(json) ?? new();

        // Создаю модель значения
        ActionStateSerializableValue value = new();

        // Очищаю списки
        Info.Clear();
        Warnings.Clear();
        Errors.Clear();

        // Для каждого информационного сообщения
        foreach (string infoString in itemSerializable.Info)
        {
            // - создаю сообщение
            ActionStateMessage info = new();
            // - десериализую в него данные из модели
            info.Deserialize(infoString);
            // - добавляю в список
            Info.Add(info);
        }
        foreach (string warningString in itemSerializable.Warnings)
        {
            // - создаю сообщение
            ActionStateMessage warning = new();
            // - десериализую в него данные из модели
            warning.Deserialize(warningString);
            // - добавляю в список
            Info.Add(warning);
        }
        foreach (string errorString in itemSerializable.Errors)
        {
            // - создаю сообщение
            ActionStateMessage error = new();
            // - десериализую в него данные из модели
            error.Deserialize(errorString);
            // - добавляю в список
            Info.Add(error);
        }

        // Десериализую данные значения из модели
        value.Deserialize(itemSerializable.Value);

        // Получаю значение
        Value = value.SetSerializedValue<T>();
    }
    #endregion
}